@@ -290,23 +290,24 @@ handle_close (XdpImplRequest *object,
 static void
 update_preview_cb (GtkFileChooser *file_chooser, gpointer data)
 {
-  GtkWidget *preview;
-  char *filename;
-  GdkPixbuf *pixbuf;
-  gboolean have_preview;
+  GtkWidget *preview = GTK_WIDGET (data);
+  g_autofree char *filename = NULL;
+  g_autoptr(GdkPixbuf) pixbuf = NULL;
 
-  preview = GTK_WIDGET (data);
   filename = gtk_file_chooser_get_preview_filename (file_chooser);
+  if (filename)
+    pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
 
-  pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
-  have_preview = (pixbuf != NULL);
-  g_free (filename);
+  if (pixbuf)
+    {
+      g_autoptr(GdkPixbuf) tmp = NULL;
+
+      tmp = gdk_pixbuf_apply_embedded_orientation (pixbuf);
+      g_set_object (&pixbuf, tmp);
+    }
 
   gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
-  if (pixbuf)
-    g_object_unref (pixbuf);
-
-  gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
+  gtk_file_chooser_set_preview_widget_active (file_chooser, pixbuf != NULL);
 }
 
 static gboolean
