@@ -2,10 +2,12 @@
 #include "appchooserdialog.h"
 
 static void
-done_cb (GtkWidget *dialog,
-         GAppInfo  *info,
-         gpointer   data)
+close_cb (AppChooserDialog *dialog,
+          gpointer   data)
 {
+        GAppInfo *info;
+
+        info = app_chooser_dialog_get_info (dialog);
         if (info)
                 g_print ("%s\n", g_app_info_get_id (info));
         else
@@ -22,8 +24,12 @@ main (int argc, char *argv[])
         const char **apps;
         int i;
         const char *default_id = NULL;
+        const char *content_type = NULL;
+        const char *filename = NULL;
         GOptionEntry entries[] = {
           { "default", 0, 0, G_OPTION_ARG_STRING, &default_id, "The default choice", "ID" },
+          { "content-type", 0, 0, G_OPTION_ARG_STRING, &content_type, "The content type", "TYPE" },
+          { "filename", 0, 0, G_OPTION_ARG_STRING, &filename, "The filename", "FILE" },
           { NULL, }
         };
 
@@ -34,8 +40,8 @@ main (int argc, char *argv[])
                 apps[i] = argv[i + 1];
         apps[argc - 1] = NULL;
 
-        window = GTK_WIDGET (app_chooser_dialog_new (apps, default_id, "Cancel", "Select", "Open With", "A heading goes here"));
-        g_signal_connect (window, "done", G_CALLBACK (done_cb), NULL);
+        window = GTK_WIDGET (app_chooser_dialog_new (apps, default_id, content_type, filename));
+        g_signal_connect (window, "close", G_CALLBACK (close_cb), NULL);
 
         gtk_widget_show (window);
 
